// DevEnvDDE.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>
#include <ddeml.h>

#define VS6_INSTANCE_NAME "msdev"
#define NET_INSTANCE_NAME "devenv"
#define NET2003_INSTANCE_NAME "VisualStudio.7.1"


static DWORD dwInstId=0L;   /* address of instance identifier       */
	
enum EVisualStudioType{
	eVS6,
	eVSNET,
	eVSNET2003
};

HDDEDATA CALLBACK DdeCallback(UINT type,      /* transaction type     */
		UINT fmt,       /* clipboard data format        */
		HCONV hconv,    /* handle of conversation       */
		HSZ hsz1,       /* handle of string     */
		HSZ hsz2,       /* handle of string     */
		HDDEDATA hData, /* handle of global memory object       */
		DWORD dwData1,  /* transaction-specific data    */
		DWORD dwData2)  /* transaction-specific data    */
{

	switch (type)
	{
		case XTYP_DISCONNECT:

			printf("DDE Callback receiving XTYP_DISCONNECT");
			break;

		case XTYP_ERROR:
			printf("DDEML fatal error !",0);
	}

	return(0);   
}

void DdeSend(HCONV hconvConv,LPSTR lpszCommand,DWORD dwSize)
{
	DdeClientTransaction ((LPBYTE)lpszCommand,
			dwSize,
			hconvConv,
			0L,
			CF_TEXT,
			XTYP_EXECUTE,
			5000,
			0L);

	//printf("DDEError: %u", DdeGetLastError(dwInstId));
}

void MSDEV_OpenFile(char* pszFileName, EVisualStudioType vsType )
{
	UINT  uRC;           /* return code */      
	DWORD afCmd;         /* array of command and filter flags    */
	DWORD uRes = 0;          /* reserved     */

	HCONV hconvConv;
	HSZ   hszService;
	HSZ   hszTopic;
	char  acCmd[240];
	char  acWinDir[240];

	FARPROC lpfnDdeCallback=NULL;
	UINT uErrCode=0;

	GetWindowsDirectory(acWinDir,MAX_PATH);

	if (!(lpfnDdeCallback=MakeProcInstance((FARPROC)DdeCallback,hiDLL))){
		printf("MakeProcInstance() error %u",uErrCode);
		return;
	}

	afCmd=APPCMD_CLIENTONLY;

	uRC=DdeInitialize((DWORD FAR*)  &dwInstId,
			(PFNCALLBACK)  lpfnDdeCallback,
			afCmd,
			uRes);

	if (DMLERR_NO_ERROR!=uRC)
	{
		uErrCode=DdeGetLastError(dwInstId);
		printf("DdeInitialize() error %u", uErrCode);
		return;
	} 

	const char *vsServiceName;
	switch(vsType){
		case eVS6: vsServiceName = VS6_INSTANCE_NAME; break;
		case eVSNET: vsServiceName = NET_INSTANCE_NAME; break;
		case eVSNET2003: vsServiceName = NET2003_INSTANCE_NAME; break;
		default:
			printf("The requested visual studio instance is not supported");
			return;
	}

	hszService = DdeCreateStringHandle(dwInstId,vsServiceName, CP_WINANSI);
	hszTopic   = DdeCreateStringHandle(dwInstId,"system",CP_WINANSI);


	if (!(hconvConv=DdeConnect(dwInstId,hszService,hszTopic,0L)))
	{
		uErrCode=DdeGetLastError(dwInstId);   
		printf("DdeConnect() error %u",uErrCode);
		return;
	} 


	wsprintf(acCmd,"Open(\"%s\")",pszFileName);
	DdeSend(hconvConv,acCmd,(DWORD)strlen(acCmd)+1);

	DdeFreeStringHandle(dwInstId,hszService);    
	DdeFreeStringHandle(dwInstId,hszTopic);    

	DdeDisconnect(hconvConv);
	DdeUninitialize(dwInstId);

	FreeProcInstance(lpfnDdeCallback);

}


void usage()
{
	        // 80 characters /////////////////////////////////////////////////////////////// 
	printf("DevEnvDDE -- open a file in an existing Visual Studio instance.\n");
	printf("Usage:\n");
	printf("    DevEnvDDE <file path> [VS6|VSNET|VSNET2003]\n");
	printf(" Will default to open the file in Visual Studio .NET 2003\n");
}

int _tmain(int argc, _TCHAR* argv[])
{
	if( argc <= 1 ){
		usage();
		return 1;
	}

	EVisualStudioType vsType = eVSNET2003;
	if( argc > 2 ){
		if(stricmp(argv[2], "VS6") == 0){
			vsType = eVS6;
		}else if(stricmp(argv[2], "VSNET") == 0){
			vsType = eVSNET;
		}else if(stricmp(argv[2], "VSNET2003") == 0){
			vsType = eVSNET2003;
		}else{
			usage();
			return 1;
		}
	}

	MSDEV_OpenFile(argv[1], vsType);
	return 0;
}

